#ifndef BASEPIN_H
#define BASEPIN_H

#include "Utils.h"

class BaseFilter;

class BasePin : public IPin {
	BasePin(const BasePin&);
	BasePin& operator=(const BasePin&);
public:
	BasePin();
	~BasePin();

	LPCWSTR GetId() const { return mpId; }
	IPin *GetConnectedPin() const { return mpConnectedPin; }
	const AM_MEDIA_TYPE *GetConnectedMediaType() const { return mpConnectedMediaType; }

	void SetPinInfo(BaseFilter *pFilter, LPCWSTR id);

	virtual HRESULT Activate();
	virtual void Deactivate();

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE Disconnect();
	HRESULT STDMETHODCALLTYPE ConnectedTo(IPin **pPin);
	HRESULT STDMETHODCALLTYPE ConnectionMediaType(AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryPinInfo(PIN_INFO *pInfo);
	HRESULT STDMETHODCALLTYPE QueryId(LPWSTR *Id);
	HRESULT STDMETHODCALLTYPE EnumMediaTypes(IEnumMediaTypes **ppEnum);
	HRESULT STDMETHODCALLTYPE QueryInternalConnections(IPin **apPin, ULONG *nPin);
	HRESULT STDMETHODCALLTYPE NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate);

protected:
	LPCWSTR mpId;

	IPin *mpConnectedPin;
	MediaTypePtr mpConnectedMediaType;
	BaseFilter *mpFilter;

	Lock mLock;
};

class InputPin : public BasePin {
public:
	InputPin();
	~InputPin();

	HRESULT STDMETHODCALLTYPE Connect(IPin *pReceivePin, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE ReceiveConnection(IPin *pConnector, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE QueryDirection(PIN_DIRECTION *pPinDir);
};

class OutputPin : public BasePin {
public:
	OutputPin();
	~OutputPin();

	HRESULT STDMETHODCALLTYPE Connect(IPin *pReceivePin, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE ReceiveConnection(IPin *pConnector, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE QueryDirection(PIN_DIRECTION *pPinDir);
};

#endif
